@@ -2,7 +2,7 @@
 // Description								/*{{{*/
 /* ######################################################################
 
-   Package Manager - Abstacts the package manager
+   Package Manager - Abstracts the package manager
 
    Three steps are 
      - Aquiration of archives (stores the list of final file names)
@@ -27,6 +27,7 @@
 #include <apt-pkg/macros.h>
 #include <apt-pkg/pkgcache.h>
 
+#include <memory>
 #include <set>
 #include <string>
 
@@ -52,9 +53,9 @@ class APT_PUBLIC pkgPackageManager : protected pkgCache::Namespace
    static bool SigINTStop;
    
    protected:
-   std::string *FileNames;
+   std::unique_ptr<std::string[]>FileNames;
    pkgDepCache &Cache;
-   pkgOrderList *List;
+   std::unique_ptr<pkgOrderList> List;
    bool Debug;
    bool NoImmConfigure;
    bool ImmConfigureAll;
@@ -78,10 +79,10 @@ class APT_PUBLIC pkgPackageManager : protected pkgCache::Namespace
    
    // Install helpers
    bool ConfigureAll();
-   bool SmartConfigure(PkgIterator Pkg, int const Depth) APT_MUSTCHECK;
-   bool SmartUnPack(PkgIterator Pkg, bool const Immediate = true, int const Depth = 0) APT_MUSTCHECK;
-   bool SmartRemove(PkgIterator Pkg) APT_MUSTCHECK;
-   bool EarlyRemove(PkgIterator Pkg, DepIterator const * const Dep) APT_MUSTCHECK;
+   [[nodiscard]] bool SmartConfigure(PkgIterator Pkg, int Depth);
+   [[nodiscard]] bool SmartUnPack(PkgIterator Pkg, bool Immediate = true, int Depth = 0);
+   [[nodiscard]] bool SmartRemove(PkgIterator Pkg);
+   [[nodiscard]] bool EarlyRemove(PkgIterator Pkg, DepIterator const *Dep);
 
    // The Actual installation implementation
    virtual bool Install(PkgIterator /*Pkg*/,std::string /*File*/) {return false;};
@@ -130,9 +131,9 @@ class APT_PUBLIC pkgPackageManager : protected pkgCache::Namespace
    private:
    void * const d;
    enum APT_HIDDEN SmartAction { UNPACK_IMMEDIATE, UNPACK, CONFIGURE };
-   APT_HIDDEN bool NonLoopingSmart(SmartAction const action, pkgCache::PkgIterator &Pkg,
-      pkgCache::PkgIterator DepPkg, int const Depth, bool const PkgLoop,
-      bool * const Bad, bool * const Changed) APT_MUSTCHECK;
+   [[nodiscard]] APT_HIDDEN bool NonLoopingSmart(SmartAction action, pkgCache::PkgIterator &Pkg,
+						 pkgCache::PkgIterator DepPkg, int Depth, bool PkgLoop,
+						 bool *Bad, bool *Changed);
 };
 
 #endif
