@@ -7,18 +7,65 @@
  * SPDX-License-Identifier: GPL-2.0+
  */
 
+#include <cassert>
+#include <memory>
 #include <optional>
 #include <queue>
 #include <vector>
 
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/depcache.h>
+#include <apt-pkg/edsp.h>
 #include <apt-pkg/pkgcache.h>
 #include <apt-pkg/policy.h>
 
+template <typename T> struct always_false : std::false_type {};
+
 namespace APT
 {
 
+/**
+ * \brief A simple mapping from objects in the cache to user-defined types.
+ *
+ * This default initializes an array with the specified value type for each
+ * object in the cache of that type.
+ */
+template <typename K, typename V, bool fast = false>
+class ContiguousCacheMap
+{
+   V *data_; // Avoid std::unique_ptr() as it may check that it's non-null.
+
+   public:
+   ContiguousCacheMap(pkgCache &cache)
+   {
+      static_assert(std::is_constructible_v<V>);
+      if constexpr (fast)
+      {
+	 static_assert(std::is_trivially_constructible_v<V>);
+	 static_assert(std::is_trivially_destructible_v<V>);
+      }
+
+      size_t size;
+      if constexpr (std::is_same_v<K, pkgCache::Version>)
+	 size = cache.Head().VersionCount;
+      else if constexpr (std::is_same_v<K, pkgCache::Package>)
+	 size = cache.Head().PackageCount;
+      else
+	 static_assert(always_false<K>::value, "Cannot construct map for key type");
+
+      data_ = new V[size]{};
+   }
+   V &operator[](const K *key) { return data_[key->ID]; }
+   const V &operator[](const K *key) const { return data_[key->ID]; }
+   ~ContiguousCacheMap() { delete[] data_; }
+};
+
+/**
+ * \brief A version of ContiguousCacheMap that ensures allocation and deallocation is trivial.
+ */
+template <typename K, typename V>
+using FastContiguousCacheMap = ContiguousCacheMap<K, V, true>;
+
 /*
  * \brief APT 3.0 solver
  *
@@ -36,8 +83,10 @@ class Solver
    struct Var;
    struct CompareProviders3;
    struct State;
+   struct Clause;
    struct Work;
    struct Solved;
+   friend struct std::hash<APT::Solver::Var>;
 
    // \brief Groups of works, these are ordered.
    //
@@ -73,7 +122,10 @@ class Solver
       // allowing a simple implementation of an autoremoval code.
       UpgradeAuto,
       KeepAuto,
-      ObsoleteAuto
+      ObsoleteAuto,
+
+      // Satisfy optional dependencies that were previously satisfied but won't otherwise be installed
+      SatisfySuggests,
    };
 
    // \brief Type to record depth at. This may very well be a 16-bit
@@ -91,29 +143,58 @@ class Solver
    pkgCache &cache;
    // Policy is needed for determining candidate version.
    pkgDepCache::Policy &policy;
+   // Root state
+   std::unique_ptr<State> rootState;
    // States for packages
-   std::vector<State> pkgStates{};
+   ContiguousCacheMap<pkgCache::Package, State> pkgStates;
    // States for versions
-   std::vector<State> verStates{};
+   ContiguousCacheMap<pkgCache::Version, State> verStates;
 
    // \brief Helper function for safe access to package state.
-   inline State &operator[](pkgCache::Package *P)
+   inline State &operator[](const pkgCache::Package *P)
    {
-      return pkgStates[P->ID];
+      return pkgStates[P];
+   }
+   inline const State &operator[](const pkgCache::Package *P) const
+   {
+      return pkgStates[P];
    }
 
    // \brief Helper function for safe access to version state.
-   inline State &operator[](pkgCache::Version *V)
+   inline State &operator[](const pkgCache::Version *V)
    {
-      return verStates[V->ID];
+      return verStates[V];
+   }
+   inline const State &operator[](const pkgCache::Version *V) const
+   {
+      return verStates[V];
    }
    // \brief Helper function for safe access to either state.
    inline State &operator[](Var r);
+   inline const State &operator[](Var r) const;
 
-   mutable std::vector<char> pkgObsolete;
-   bool Obsolete(pkgCache::PkgIterator pkg) const;
+   mutable FastContiguousCacheMap<pkgCache::Package, char> pkgObsolete;
+   // \brief Check if package is obsolete.
+   // \param AllowManual controls whether manual packages can be obsolete
+   bool Obsolete(pkgCache::PkgIterator pkg, bool AllowManual=false) const;
    bool ObsoletedByNewerSourceVersion(pkgCache::VerIterator cand) const;
 
+   mutable FastContiguousCacheMap<pkgCache::Version, short> priorities;
+   short GetPriority(pkgCache::VerIterator ver) const
+   {
+      if (priorities[ver] == 0)
+	 priorities[ver] = policy.GetPriority(ver);
+      return priorities[ver];
+   }
+
+   mutable ContiguousCacheMap<pkgCache::Package, pkgCache::VerIterator> candidates;
+   pkgCache::VerIterator GetCandidateVer(pkgCache::PkgIterator pkg) const
+   {
+      if (candidates[pkg].end())
+	 candidates[pkg] = policy.GetCandidateVer(pkg);
+      return candidates[pkg];
+   }
+
    // \brief Heap of the remaining work.
    //
    // We are using an std::vector with std::make_heap(), std::push_heap(),
@@ -121,8 +202,6 @@ class Solver
    // be able to iterate over the queued work and see if a choice would
    // invalidate any work.
    heap<Work> work{};
-   // \brief Whether RescoreWork() actually needs to rescore the work.
-   bool needsRescore{false};
 
    // \brief Backlog of solved work.
    //
@@ -134,73 +213,109 @@ class Solver
 
    // \brief Propagation queue
    std::queue<Var> propQ;
+   // \brief Discover variables
+   std::queue<Var> discoverQ;
 
    // \brief Current decision level.
    //
    // This is an index into the solved vector.
    std::vector<depth_type> choices{};
 
+   // \brief The time we called Solve()
+   time_t startTime;
+
+   EDSP::Request::Flags requestFlags;
    /// Various configuration options
+   std::string version{_config->Find("APT::Solver", "3.0")};
    // \brief Debug level
    int debug{_config->FindI("Debug::APT::Solver")};
    // \brief If set, we try to keep automatically installed packages installed.
-   bool KeepAuto{not _config->FindB("APT::Get::AutomaticRemove")};
+   bool KeepAuto{version == "3.0" || not _config->FindB("APT::Get::AutomaticRemove")};
+   // \brief Determines if we are in upgrade mode.
+   bool IsUpgrade{_config->FindB("APT::Solver::Upgrade", requestFlags &EDSP::Request::UPGRADE_ALL)};
    // \brief If set, removals are allowed.
-   bool AllowRemove{_config->FindB("APT::Solver::Remove", true)};
+   bool AllowRemove{_config->FindB("APT::Solver::Remove", not(requestFlags & EDSP::Request::FORBID_REMOVE))};
+   // \brief If set, removal of manual packages is allowed.
+   bool AllowRemoveManual{AllowRemove && _config->FindB("APT::Solver::RemoveManual", false)};
    // \brief If set, installs are allowed.
-   bool AllowInstall{_config->FindB("APT::Solver::Install", true)};
+   bool AllowInstall{_config->FindB("APT::Solver::Install", not(requestFlags & EDSP::Request::FORBID_NEW_INSTALL))};
    // \brief If set, we use strict pinning.
    bool StrictPinning{_config->FindB("APT::Solver::Strict-Pinning", true)};
+   // \brief If set, we install missing recommends and pick new best packages.
+   bool FixPolicyBroken{_config->FindB("APT::Get::Fix-Policy-Broken")};
+   // \brief If set, we use strict pinning.
+   bool DeferVersionSelection{_config->FindB("APT::Solver::Defer-Version-Selection", true)};
+   // \brief If set, we use strict pinning.
+   int Timeout{_config->FindI("APT::Solver::Timeout", 10)};
 
+   // \brief Keep recommends installed
+   bool KeepRecommends{_config->FindB("APT::AutoRemove::RecommendsImportant", true)};
+   // \brief Keep suggests installed
+   bool KeepSuggests{_config->FindB("APT::AutoRemove::SuggestsImportant", true)};
+
+   // \brief Discover a variable, translating the underlying dependencies to the SAT presentation
+   //
+   // This does a breadth-first search of the entire dependency tree of var,
+   // utilizing the discoverQ above.
+   void Discover(Var var);
+   // \brief Link a clause into the watchers
+   const Clause *RegisterClause(Clause &&clause);
    // \brief Enqueue dependencies shared by all versions of the package.
-   bool EnqueueCommonDependencies(pkgCache::PkgIterator Pkg);
-   // \brief Reject reverse dependencies. Must call std::make_heap() after.
-   bool RejectReverseDependencies(pkgCache::VerIterator Ver);
-   // \brief Enqueue a single or group
-   bool EnqueueOrGroup(pkgCache::DepIterator start, pkgCache::DepIterator end, Var reason);
+   void RegisterCommonDependencies(pkgCache::PkgIterator Pkg);
+
+   // \brief Translate an or group into a clause object
+   [[nodiscard]] Clause TranslateOrGroup(pkgCache::DepIterator start, pkgCache::DepIterator end, Var reason);
    // \brief Propagate all pending propagations
-   bool Propagate();
-   // \brief Propagate a "true" value of a variable
-   bool PropagateInstall(Var var);
-   // \brief Propagate a rejection of a variable
-   bool PropagateReject(Var var);
+   [[nodiscard]] bool Propagate();
 
    // \brief Return the current depth (choices.size() with casting)
    depth_type depth()
    {
       return static_cast<depth_type>(choices.size());
    }
+   inline Var bestReason(Clause const *clause, Var var) const;
 
    public:
    // \brief Create a new decision level.
    void Push(Work work);
    // \brief Revert to the previous decision level.
-   bool Pop();
+   [[nodiscard]] bool Pop();
    // \brief Undo a single assignment / solved work item
    void UndoOne();
    // \brief Add work to our work queue.
-   void AddWork(Work &&work);
-   // \brief Rescore the work after a reject or a pop
-   void RescoreWorkIfNeeded();
+   [[nodiscard]] bool AddWork(Work &&work);
 
    // \brief Basic solver initializer. This cannot fail.
-   Solver(pkgCache &Cache, pkgDepCache::Policy &Policy);
+   Solver(pkgCache &Cache, pkgDepCache::Policy &Policy, EDSP::Request::Flags requestFlags);
 
    // Assume that the variable is decided as specified.
-   bool Assume(Var var, bool decision, Var reason);
+   [[nodiscard]] bool Assume(Var var, bool decision, const Clause *reason = nullptr);
    // Enqueue a decision fact
-   bool Enqueue(Var var, bool decision, Var reason);
+   [[nodiscard]] bool Enqueue(Var var, bool decision, const Clause *reason = nullptr);
 
    // \brief Apply the selections from the dep cache to the solver
-   bool FromDepCache(pkgDepCache &depcache);
+   [[nodiscard]] bool FromDepCache(pkgDepCache &depcache);
    // \brief Apply the solver result to the depCache
-   bool ToDepCache(pkgDepCache &depcache);
+   [[nodiscard]] bool ToDepCache(pkgDepCache &depcache) const;
 
    // \brief Solve the dependencies
-   bool Solve();
+   [[nodiscard]] bool Solve();
 
    // Print dependency chain
-   std::string WhyStr(Var reason);
+   std::string WhyStr(Var reason) const;
+
+   /**
+    * \brief Print a long reason string
+    *
+    * Print a reason as to why `rclause` implies `decision` for the variable `var`.
+    *
+    * \param var The variable to print the reason for
+    * \param decision The assumed decision to print the reason for (may be different from actual decision if rclause is specified)
+    * \param rclause The clause that caused this variable to be marked (or would be marked)
+    * \param prefix A prefix, for indentation purposes, as this is recursive
+    * \param seen A set of seen objects such that the output does not repeat itself (not for safety, it is acyclic)
+    */
+   std::string LongWhyStr(Var var, bool decision, const Clause *rclause, std::string prefix, std::unordered_set<Var> &seen) const;
 };
 
 }; // namespace APT
@@ -215,38 +330,44 @@ class Solver
  */
 struct APT::Solver::Var
 {
-   uint32_t IsVersion : 1;
-   uint32_t MapPtr : 31;
+   uint32_t value;
 
-   Var() : IsVersion(0), MapPtr(0) {}
-   explicit Var(pkgCache::PkgIterator const &Pkg) : IsVersion(0), MapPtr(Pkg.MapPointer()) {}
-   explicit Var(pkgCache::VerIterator const &Ver) : IsVersion(1), MapPtr(Ver.MapPointer()) {}
+   explicit constexpr Var(uint32_t value = 0) : value{value} {}
+   explicit Var(pkgCache::PkgIterator const &Pkg) : value(uint32_t(Pkg.MapPointer()) << 1) {}
+   explicit Var(pkgCache::VerIterator const &Ver) : value(uint32_t(Ver.MapPointer()) << 1 | 1) {}
+
+   inline constexpr bool isVersion() const { return value & 1; }
+   inline constexpr uint32_t mapPtr() const { return value >> 1; }
 
    // \brief Return the package, if any, otherwise 0.
    map_pointer<pkgCache::Package> Pkg() const
    {
-      return IsVersion ? 0 : map_pointer<pkgCache::Package>{(uint32_t)MapPtr};
+      return isVersion() ? 0 : map_pointer<pkgCache::Package>{mapPtr()};
    }
    // \brief Return the version, if any, otherwise 0.
    map_pointer<pkgCache::Version> Ver() const
    {
-      return IsVersion ? map_pointer<pkgCache::Version>{(uint32_t)MapPtr} : 0;
+      return isVersion() ? map_pointer<pkgCache::Version>{mapPtr()} : 0;
    }
    // \brief Return the package iterator if storing a package, or an empty one
    pkgCache::PkgIterator Pkg(pkgCache &cache) const
    {
-      return IsVersion ? pkgCache::PkgIterator() : pkgCache::PkgIterator(cache, cache.PkgP + Pkg());
+      return isVersion() ? pkgCache::PkgIterator() : pkgCache::PkgIterator(cache, cache.PkgP + Pkg());
    }
    // \brief Return the version iterator if storing a package, or an empty end.
    pkgCache::VerIterator Ver(pkgCache &cache) const
    {
-      return IsVersion ? pkgCache::VerIterator(cache, cache.VerP + Ver()) : pkgCache::VerIterator();
+      return isVersion() ? pkgCache::VerIterator(cache, cache.VerP + Ver()) : pkgCache::VerIterator();
+   }
+   // \brief Return a package, cast from version if needed
+   pkgCache::PkgIterator CastPkg(pkgCache &cache) const
+   {
+      return isVersion() ? Ver(cache).ParentPkg() : Pkg(cache);
    }
    // \brief Check if there is no reason.
-   bool empty() const
-   {
-      return IsVersion == 0 && MapPtr == 0;
-   }
+   constexpr bool empty() const { return value == 0; }
+   constexpr bool operator!=(Var const other) const { return value != other.value; }
+   constexpr bool operator==(Var const other) const { return value == other.value; }
 
    std::string toString(pkgCache &cache) const
    {
@@ -258,6 +379,33 @@ struct APT::Solver::Var
    }
 };
 
+/**
+ * \brief A single clause
+ *
+ * A clause is a normalized, expanded dependency, translated into an implication
+ * in terms of Var objects, that is, `reason -> solutions[0] | ... | solutions[n]`
+ */
+struct APT::Solver::Clause
+{
+   // \brief Underyling dependency
+   pkgCache::Dependency *dep = nullptr;
+   // \brief Var for the work
+   Var reason;
+   // \brief The group we are in
+   Group group;
+   // \brief Possible solutions to this task, ordered in order of preference.
+   std::vector<Var> solutions{};
+   // \brief An optional clause does not need to be satisfied
+   bool optional;
+
+   // \brief A negative clause negates the solutions, that is X->A|B you get X->!(A|B), aka X->!A&!B
+   bool negative;
+
+   inline Clause(Var reason, Group group, bool optional = false, bool negative = false) : reason(reason), group(group), optional(optional), negative(negative) {}
+
+   std::string toString(pkgCache &cache, bool pretty = false) const;
+};
+
 /**
  * \brief A single work item
  *
@@ -270,37 +418,27 @@ struct APT::Solver::Var
  */
 struct APT::Solver::Work
 {
-   // \brief Var for the work
-   Var reason;
+   const Clause *clause;
+
    // \brief The depth at which the item has been added
    depth_type depth;
-   // \brief The group we are in
-   Group group;
-   // \brief Possible solutions to this task, ordered in order of preference.
-   std::vector<pkgCache::Version *> solutions{};
 
    // This is a union because we only need to store the choice we made when adding
    // to the choice vector, and we don't need the size of valid choices in there.
    union
    {
       // The choice we took
-      pkgCache::Version *choice;
+      Var choice;
       // Number of valid choices
-      size_t size;
+      size_t size{0};
    };
 
-   // \brief Whether this is an optional work item, they will be processed last
-   bool optional;
-   // \brief Whether this is an ugprade
-   bool upgrade;
    // \brief This item should be removed from the queue.
-   bool erased;
+   bool erased{false};
 
    bool operator<(APT::Solver::Work const &b) const;
-   // \brief Dump the work item to std::cerr
-   void Dump(pkgCache &cache);
-
-   inline Work(Var reason, depth_type depth, Group group, bool optional = false, bool upgrade = false) : reason(reason), depth(depth), group(group), size(0), optional(optional), upgrade(upgrade), erased(false) {}
+   std::string toString(pkgCache &cache) const;
+   inline Work(const Clause *clause, depth_type depth) : clause(clause), depth(depth) {}
 };
 
 // \brief This essentially describes the install state in RFC2119 terms.
@@ -314,17 +452,6 @@ enum class APT::Solver::Decision : uint16_t
    MUSTNOT,
 };
 
-// \brief Hints for the solver about the item.
-enum class APT::Solver::Hint : uint16_t
-{
-   // \brief We have not made a choice about the package yet
-   NONE,
-   // \brief This package was listed as a Recommends of a must package,
-   SHOULD,
-   // \brief This package was listed as a Suggests of a must-not package
-   MAY,
-};
-
 /**
  * \brief The solver state
  *
@@ -344,7 +471,7 @@ struct APT::Solver::State
    // doesn't increase to unwind.
    //
    // Vars < 0 are package ID, reasons > 0 are version IDs.
-   Var reason{};
+   const Clause *reason{};
 
    // \brief The depth at which the decision has been taken
    depth_type depth{0};
@@ -352,8 +479,19 @@ struct APT::Solver::State
    // \brief This essentially describes the install state in RFC2119 terms.
    Decision decision{Decision::NONE};
 
-   // \brief Any hint.
-   Hint hint{Hint::NONE};
+   // \brief Flags.
+   struct
+   {
+      bool discovered{};
+      bool manual{};
+   } flags;
+
+   static_assert(sizeof(flags) <= sizeof(int));
+
+   // \brief Clauses owned by this package/version
+   std::vector<std::unique_ptr<Clause>> clauses;
+   // \brief Reverse clauses, that is dependencies (or conflicts) from other packages on this one
+   std::vector<const Clause *> rclauses;
 };
 
 /**
@@ -376,6 +514,18 @@ inline APT::Solver::State &APT::Solver::operator[](Var r)
       return (*this)[cache.PkgP + P];
    if (auto V = r.Ver())
       return (*this)[cache.VerP + V];
-
-   abort();
+   return *rootState.get();
 }
+
+inline const APT::Solver::State &APT::Solver::operator[](Var r) const
+{
+   return const_cast<Solver &>(*this)[r];
+}
+
+// Custom specialization of std::hash can be injected in namespace std.
+template <>
+struct std::hash<APT::Solver::Var>
+{
+   std::hash<decltype(APT::Solver::Var::value)> hash_value;
+   std::size_t operator()(const APT::Solver::Var &v) const noexcept { return hash_value(v.value); }
+};
