@@ -123,7 +123,7 @@ class PackagesWriter : public FTWScanner
    inline bool ReadOverride(string const &File) {return Over.ReadOverride(File);};
    inline bool ReadExtraOverride(string const &File)
       {return Over.ReadExtraOverride(File);};
-   virtual bool DoPackage(string FileName) APT_OVERRIDE;
+   bool DoPackage(string FileName) override;
 
    PackagesWriter(FileFd * const Output, TranslationWriter * const TransWriter, string const &DB,
                   string const &Overrides,
@@ -146,7 +146,7 @@ class ContentsWriter : public FTWScanner
    string Prefix;
 
    bool DoPackage(string FileName,string Package);
-   virtual bool DoPackage(string FileName) APT_OVERRIDE
+   bool DoPackage(string FileName) override
              {return DoPackage(FileName,string());};
    bool ReadFromPkgs(string const &PkgFile,string const &PkgCompress);
 
@@ -176,7 +176,7 @@ class SourcesWriter : public FTWScanner
    string DirStrip;
    struct CacheDB::Stats &Stats;
 
-   virtual bool DoPackage(string FileName) APT_OVERRIDE;
+   bool DoPackage(string FileName) override;
 
    SourcesWriter(FileFd * const Output, string const &DB,string const &BOverrides,string const &SOverrides,
 		 string const &ExtOverrides=string());
@@ -187,7 +187,7 @@ class ReleaseWriter : public FTWScanner
 {
 public:
    ReleaseWriter(FileFd * const Output, string const &DB);
-   virtual bool DoPackage(string FileName) APT_OVERRIDE;
+   bool DoPackage(string FileName) override;
    void Finish();
 
    // General options
